==> metatags_import_csv-1.0.3.tar.gz: metatags_import_csv/src/Form/ImportForm.php <==
<?php

namespace Drupal\metatags_import_csv\Form;

use Drupal;
use Drupal\Core\Entity\EntityStorageInterface;
use Drupal\Core\Entity\EntityTypeManager;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\node\NodeInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 *
 */
class ImportForm extends FormBase {

  final public const IMPORT_BATCH_SIZE = 5;

  final public const CSV_SEPARATOR = ';';

  /**
   * Node Storage.
   *
   * @var \Drupal\Core\Entity\EntityStorageInterface
   */
  private readonly EntityStorageInterface $nodeStorage;

  /**
   * {@inheritdoc}
   */
  public function __construct(EntityTypeManager $entityTypeManager) {
    $this->nodeStorage = $entityTypeManager->getStorage('node');
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container): static {
    return new static($container->get('entity_type.manager'));
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'metatags_import_csv_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $form['import_metas_file'] = [
      '#type' => 'file',
      '#title' => "Fichier csv metas import",
      '#upload_validators' => [
        'file_validate_extensions' => ['csv'],
      ],
    ];

    $form['action']['submit'] = [
      '#type' => 'submit',
      '#value' => 'Import',
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $filePathname = $this->getRequest()->files->all()['files']['import_metas_file']->getPathname();

    $datas = [];
    $firstLine = TRUE;

    $handle = fopen($filePathname, 'r');

    if ($handle != FALSE) {
      while (($data = fgetcsv($handle, 0, self::CSV_SEPARATOR)) != FALSE) {
        if ($firstLine) {
          $firstLine = FALSE;
          continue;
        }

        $datas[] = $data;
      }
    }

    $batch = [
      'title' => $this->t('Import metas ...'),
      'operations' => [
        [
          '\Drupal\metatags_import_csv\Form\ImportForm::batchImport',
          [$datas],
        ],
      ],
    ];

    batch_set($batch);
  }

  /**
   * Save metas datas.
   *
   * @param array $datasImport
   *   Data for batch.
   * @param array $context
   *   Context.
   */
  public static function batchImport(
    array $datasImport,
    array &$context
  ): void {
    if (empty($context['sandbox'])) {
      $context['sandbox']['progress'] = 0;
      $context['sandbox']['max'] = count($datasImport);
      $context['logs'] = [];
    }

    $metasToImport = array_slice($datasImport,
      $context['sandbox']['progress'], self::IMPORT_BATCH_SIZE, TRUE);

    $nodeStorage = Drupal::entityTypeManager()
      ->getStorage('node');

    foreach ($metasToImport as $index => $metaData) {
      $percent = round(((int) $context['sandbox']['progress'] * 100) / (int) $context['sandbox']['max'],
        2);
      $context['message'] = "Save node metadata {$metaData[0]} -- (" . $context['sandbox']['progress'] . "/" . $context['sandbox']['max'] . " - $percent%)";
      $context['sandbox']['progress']++;

      if (empty($metaData[0])) {
        Drupal::logger('Metadatas - Import')
          ->debug(t('Line @line -- Node url is empty.',
            ['@line' => $index + 2]));
        continue;
      }

      if (empty($metaData[1])) {
        Drupal::logger('Metadatas - Import')
          ->debug(t('Line @line -- Language is empty.',
            ['@line' => $index + 2]));
        continue;
      }

      $request = \Drupal::request();
      $currentDomain = $request->getSchemeAndHttpHost() . $request->getBasePath();

      $url = str_replace($currentDomain, '', $metaData[0]);

      $params = Drupal\Core\Url::fromUri("internal:" . $url)->getRouteParameters();

      if (empty($params['node'])) {
        Drupal::logger('Metadatas - Import')
          ->debug(t('Line @line -- Node path @path doesnt exist.',
            [
              '@line' => $index + 2,
              '@path' => $metaData[0],
            ]));
        continue;
      }

      $node = $nodeStorage->load($params['node']);

      if (!$node instanceof NodeInterface) {
        Drupal::logger('Metadatas - Import')
          ->debug(t('Line @line -- Node @id not exist.',
            ['@line' => $index + 2, '@id' => $metaData[0]]));
        continue;
      }

      if (!$node->hasTranslation($metaData[1])) {
        Drupal::logger('Metadatas - Import')
          ->debug(t('Line @line -- Node  "@title" (id : @id) does\'nt have translation @langcode.',
            [
              '@line' => $index + 2,
              '@id' => $metaData[0],
              '@langcode' => $metaData[1],
              '@title' => $node->label(),
            ]));
        continue;
      }

      $node = $node->getTranslation($metaData[1]);

      if (!$node->hasField('field_metatags')) {
        Drupal::logger('Metadatas - Import')
          ->debug(t('Line @line -- Node "@title" (id : @id) does not have metatags field.',
            [
              '@line' => $index + 2,
              '@id' => $metaData[0],
              '@title' => $node->label(),
            ]));
        continue;
      }

      $metas = $node->get('field_metatags')->getValue();
      $metasUnserialize = unserialize($metas[0]['value']);

      if (!empty($metaData[3])) {
        $metasUnserialize['title'] = $metaData[3];
      }

      if (!empty($metaData[2])) {
        $metasUnserialize['description'] = $metaData[2];
      }

      $node->set('field_metatags', serialize($metasUnserialize));
      $node->save();
    }

    if ($context['sandbox']['progress'] != $context['sandbox']['max']) {
      $context['finished'] = $context['sandbox']['progress'] / $context['sandbox']['max'];
    }
  }

}

==> metatags_import_csv-1.0.4.tar.gz: metatags_import_csv/src/Form/ImportForm.php <==
<?php

namespace Drupal\metatags_import_csv\Form;

use Drupal;
use Drupal\Core\Entity\EntityStorageInterface;
use Drupal\Core\Entity\EntityTypeManager;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\node\NodeInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 *
 */
class ImportForm extends FormBase {

  final public const IMPORT_BATCH_SIZE = 5;

  final public const CSV_SEPARATOR = ';';

  /**
   * Node Storage.
   *
   * @var \Drupal\Core\Entity\EntityStorageInterface
   */
  private readonly EntityStorageInterface $nodeStorage;

  /**
   * {@inheritdoc}
   */
  public function __construct(EntityTypeManager $entityTypeManager) {
    $this->nodeStorage = $entityTypeManager->getStorage('node');
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container): static {
    return new static($container->get('entity_type.manager'));
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'metatags_import_csv_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $form['import_metas_file'] = [
      '#type' => 'file',
      '#title' => "Fichier csv metas import",
      '#upload_validators' => [
        'file_validate_extensions' => ['csv'],
      ],
    ];

    $form['action']['submit'] = [
      '#type' => 'submit',
      '#value' => 'Import',
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $filePathname = $this->getRequest()->files->all()['files']['import_metas_file']->getPathname();

    $datas = [];
    $firstLine = TRUE;

    $handle = fopen($filePathname, 'r');

    if ($handle != FALSE) {
      while (($data = fgetcsv($handle, 0, self::CSV_SEPARATOR)) != FALSE) {
        if ($firstLine) {
          $firstLine = FALSE;
          continue;
        }

        $datas[] = $data;
      }
    }

    $batch = [
      'title' => $this->t('Import metas ...'),
      'operations' => [
        [
          '\Drupal\metatags_import_csv\Form\ImportForm::batchImport',
          [$datas],
        ],
      ],
    ];

    batch_set($batch);
  }

  /**
   * Save metas datas.
   *
   * @param array $datasImport
   *   Data for batch.
   * @param array $context
   *   Context.
   */
  public static function batchImport(
    array $datasImport,
    array &$context
  ): void {
    if (empty($context['sandbox'])) {
      $context['sandbox']['progress'] = 0;
      $context['sandbox']['max'] = count($datasImport);
      $context['logs'] = [];
    }

    $metasToImport = array_slice($datasImport,
      $context['sandbox']['progress'], self::IMPORT_BATCH_SIZE, TRUE);

    $nodeStorage = Drupal::entityTypeManager()
      ->getStorage('node');

    foreach ($metasToImport as $index => $metaData) {
      $percent = round(((int) $context['sandbox']['progress'] * 100) / (int) $context['sandbox']['max'],
        2);
      $context['message'] = "Save node metadata {$metaData[0]} -- (" . $context['sandbox']['progress'] . "/" . $context['sandbox']['max'] . " - $percent%)";
      $context['sandbox']['progress']++;

      if (empty($metaData[0])) {
        Drupal::logger('Metadatas - Import')
          ->debug(t('Line @line -- Node url is empty.',
            ['@line' => $index + 2]));
        continue;
      }

      if (empty($metaData[1])) {
        Drupal::logger('Metadatas - Import')
          ->debug(t('Line @line -- Language is empty.',
            ['@line' => $index + 2]));
        continue;
      }

      $request = \Drupal::request();
      $currentDomain = $request->getSchemeAndHttpHost() . $request->getBasePath();

      if (!str_contains($metaData[0], $currentDomain)) {
        Drupal::logger('Metadatas - Import')
          ->debug(t('Line @line -- Bad protocol/domain : @path',
            [
              '@line' => $index + 2,
              '@path' => $metaData[0],
            ]));
        continue;
      }

      $url = str_replace($currentDomain, '', $metaData[0]);

      $params = Drupal\Core\Url::fromUri("internal:" . $url)->getRouteParameters();

      if (empty($params['node'])) {
        Drupal::logger('Metadatas - Import')
          ->debug(t('Line @line -- Node path @path doesnt exist.',
            [
              '@line' => $index + 2,
              '@path' => $metaData[0],
            ]));
        continue;
      }

      $node = $nodeStorage->load($params['node']);

      if (!$node instanceof NodeInterface) {
        Drupal::logger('Metadatas - Import')
          ->debug(t('Line @line -- Node @id not exist.',
            ['@line' => $index + 2, '@id' => $metaData[0]]));
        continue;
      }

      if (!$node->hasTranslation($metaData[1])) {
        Drupal::logger('Metadatas - Import')
          ->debug(t('Line @line -- Node  "@title" (id : @id) does\'nt have translation @langcode.',
            [
              '@line' => $index + 2,
              '@id' => $metaData[0],
              '@langcode' => $metaData[1],
              '@title' => $node->label(),
            ]));
        continue;
      }

      $node = $node->getTranslation($metaData[1]);

      if (!$node->hasField('field_metatags')) {
        Drupal::logger('Metadatas - Import')
          ->debug(t('Line @line -- Node "@title" (id : @id) does not have metatags field.',
            [
              '@line' => $index + 2,
              '@id' => $metaData[0],
              '@title' => $node->label(),
            ]));
        continue;
      }

      $metas = $node->get('field_metatags')->getValue();
      $metasUnserialize = unserialize($metas[0]['value']);

      if (!empty($metaData[3])) {
        $metasUnserialize['title'] = utf8_encode($metaData[3]);
      }

      if (!empty($metaData[2])) {
        $metasUnserialize['description'] = utf8_encode($metaData[2]);
      }

      $node->set('field_metatags', serialize($metasUnserialize));
      $node->save();
    }

    if ($context['sandbox']['progress'] != $context['sandbox']['max']) {
      $context['finished'] = $context['sandbox']['progress'] / $context['sandbox']['max'];
    }
  }

}
